(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

open Common

module SDFA = struct
  type ('s,'o) t = {
    m: 's Bdd.mem;
    t: 's -> 's Bdd.node span;
    o: 's -> 'o;
    state_info: Format.formatter -> 's -> unit;
    output_info: Format.formatter -> 'o -> unit
  }
  let trace ?exclude a =
    let si x = Format.asprintf "%a" a.state_info x in
    let oi o = Format.asprintf "%a" a.output_info o in
    let exclude = match exclude with None -> None | Some x -> Some (Bdd.constant a.m x) in
    let trace = Bdd.trace ?exclude (fun trace i x ->
      Trace.leaf i (si x) (oi (a.o x));
      Span.iter (fun a x' -> 
	if (match exclude with None -> true | Some y -> x'!=y) then (
	  trace x';
	  Trace.leaf_t i a (Bdd.tag x'))
      ) (a.t x)) 
    in fun x -> 
      let x = Bdd.constant a.m x in 
      Trace.entry (Bdd.tag x); trace x

end
type ('s,'o) sdfa = ('s,'o) SDFA.t

module SNFA = struct
  type ('s,'set,'o) t = {
    m: 'set Bdd.mem;
    t: 's -> 'set Bdd.node span;
    o: 's -> 'o;
    state_info: Format.formatter -> 's -> unit;
    output_info: Format.formatter -> 'o -> unit
  }
  let int_map f x = Hset.fold (fun i -> IntSet.add (f i)) x IntSet.empty
  let reindex a =
    let m = Bdd.init Hashtbl.hash (=) in
    let z = Span.empty (Bdd.constant m IntSet.empty) in
    let back = ref Hmap.empty in
    let r = ref 0 in    
    let g = ref [||] in
    let rec f x = 
      try Hmap.find x !back 
      with Not_found ->
	let i = !r in
	let t = !g in
	let l = Array.length t in
	let o = a.o x in
	back := Hmap.add x i !back;
	if i = l then (
	  g := Array.make (2*l+1) (o,z,x);
	  Array.blit t 0 !g 0 i
	); 
	incr r;
	!g.(i) <- (o, Span.map (Bdd.unary m f') (a.t x), x);
	i
    and f' x = int_map f x in
    let fst (x,_,_) = x in
    let snd (_,x,_) = x in
    let thd (_,_,x) = x in
    let t i = snd !g.(i) in
    let o i = fst !g.(i) in
    let state_info f i = a.state_info f (thd !g.(i)) in
    let output_info = a.output_info in
    { m; t; o; state_info; output_info}, f'
end
type ('s,'t,'o) snfa = ('s,'t,'o) SNFA.t

module SENFA = struct
  type t = {
    e : (int * Bdd.formula * int) list;
    t : (int * var * int) list;
    o : int set;
    size: int;
  }
  end
type senfa = SENFA.t
