open Common
open Bdd

(** Types for the various kind of manipulated automata *)

(** symbolic deterministic automata *)
module SDFA: sig
  type ('s, 'o) t = {
    m : 's mem;
    t : 's -> 's node span;
    o : 's -> 'o;
    state_info : Format.formatter -> 's -> unit;
    output_info : Format.formatter -> 'o -> unit;
  }
  val trace: ?exclude:'s -> ('s,'o) t -> ('s -> unit) 
end
type ('s, 'o) sdfa = ('s, 'o) SDFA.t

(** symbolic non-deterministic automata *)
module SNFA:  sig
  type ('s, 'set, 'o) t = {
    m : 'set mem;
    t : 's -> 'set node span;
    o : 's -> 'o;
    state_info : Format.formatter -> 's -> unit;
    output_info : Format.formatter -> 'o -> unit;
  }
  (** reindex and memoise an NFA *)
  val reindex: ('s hval,'s hset,'o) t -> (int,int_set,'o) t * ('s hset -> int_set)
end
type ('s, 't, 'o) snfa = ('s, 't, 'o) SNFA.t

(** non-deterministic automata with epsilon transitions *)
module SENFA: sig
  type t = {
    e : (int * formula * int) list;
    t : (int * var * int) list;
    o : int set;
    size: int;
  }
end
type senfa = SENFA.t
