(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

(** Symbolic algorithms for language equivalence *)

open Common
open Automata

(** Symbolic equivalence check, using queues [Q]

Candidates for [Q] include:
- fifo queues for breadth-first exploration ([Queues.BFS])
- lifo queues for depth-first exploration ([Queues.DFS])
- random queues for random exploration ([Queues.RFS])

By changing the BDD unifier, we get either the naive or 
the Hopcroft and Karp like algorithm.

Return [None] if the states were language-equivalent, or a path
leading to two states whose output value differ. (Note that the path
is reversed.)

*)
module Symbolic(Q: QUEUE): sig
  val equiv: 
    ?tracer:([`OK|`CE|`Skip] -> 's -> 's -> unit) ->
    's Bdd.unifier -> ('s,'o) sdfa -> 's -> 's -> ('s*'s*gstring_) option

  (** same algorithms, further enhanced using up to congruence *)
  val equiv_c: 
    ?tracer:([`OK|`CE|`Skip] -> int_set -> int_set -> unit) ->
    int_set Bdd.unifier -> (int_set,'o) sdfa -> int_set -> int_set -> (int_set*int_set*gstring_) option
end
